@@ -335,7 +335,6 @@ init_devices (gpointer ctx)
 
 		if (ok_devices_present == 0) {
 			GtkWidget *toplevel;
-			GtkWidget *dialog;
 
 			/* FIXME: Let's assume that there's only a problem with /dev/fd0 */
 			switch (floppies[0].status) {
@@ -563,8 +562,6 @@ main (int argc, char *argv[])
 	GtkSizeGroup *size_group;
 	GSList *l;
 	gchar *label_text;
-        gint base_size;
-
 
 	/* This is for the libhal context, should we need it */
 	gpointer ctx;
