@@ -55,8 +55,24 @@ extern pid_t getpgid (pid_t);
 gboolean row_selected_by_button_press_event;
 
 static void
-quit_application (GSearchCommandDetails * command_details)
+store_window_state_and_geometry (GSearchWindow *gsearch)
 {
+	gsearch->window_width = MAX (gsearch->window_width, MINIMUM_WINDOW_WIDTH);
+	gsearch->window_height = MAX (gsearch->window_height, MINIMUM_WINDOW_HEIGHT);
+		
+	gsearchtool_gconf_set_int ("/apps/gnome-search-tool/default_window_width",
+	                           gsearch->window_width);
+	gsearchtool_gconf_set_int ("/apps/gnome-search-tool/default_window_height",
+		                   gsearch->window_height);
+	gsearchtool_gconf_set_boolean ("/apps/gnome-search-tool/default_window_maximized",
+	                               gsearch->is_window_maximized);
+}
+
+static void
+quit_application (GSearchWindow * gsearch)
+{
+	GSearchCommandDetails * command_details = gsearch->command_details;
+	
 	if (command_details->command_status == RUNNING) {
 #ifdef HAVE_GETPGID
 		pid_t pgid;
@@ -76,6 +92,7 @@ quit_application (GSearchCommandDetails * command_details)
 #endif
 		wait (NULL);
 	}
+	store_window_state_and_geometry (gsearch);
 	gtk_main_quit ();
 }
 
@@ -83,7 +100,7 @@ void
 die_cb (GnomeClient * client,
 	gpointer data)
 {
-	quit_application ((GSearchCommandDetails *) data);
+	quit_application ((GSearchWindow *) data);
 }
 
 void
@@ -91,14 +108,14 @@ quit_cb (GtkWidget * widget,
          GdkEvent * event,
 	 gpointer data)
 {
-	quit_application ((GSearchCommandDetails *) data);
+	quit_application ((GSearchWindow *) data);
 }
 
 void
 click_close_cb (GtkWidget * widget,
                 gpointer data)
 {
-	quit_application ((GSearchCommandDetails *) data);
+	quit_application ((GSearchWindow *) data);
 }
 
 void
@@ -582,6 +599,33 @@ open_folder_cb (GtkAction * action,
 	g_list_free (list);
 }
 
+void
+file_changed_cb (GnomeVFSMonitorHandle * handle,
+                 const gchar * monitor_uri,
+                 const gchar * info_uri,
+                 GnomeVFSMonitorEventType event_type,
+                 gpointer data)
+{
+	GSearchMonitor * monitor = data;
+	GSearchWindow * gsearch = monitor->gsearch;
+	GtkTreeModel * model;
+	GtkTreePath * path;
+	GtkTreeIter iter;
+	
+	switch (event_type) {
+	case GNOME_VFS_MONITOR_EVENT_DELETED:
+		path = gtk_tree_row_reference_get_path (monitor->reference);
+		model = gtk_tree_row_reference_get_model (monitor->reference);
+		gtk_tree_model_get_iter (model, &iter, path);
+		tree_model_iter_free_monitor (model, NULL, &iter, NULL);
+		gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+		update_search_counts (gsearch);
+		break;
+	default:
+		break;
+	}	    	
+}
+
 static void
 display_dialog_could_not_move_to_trash (GtkWidget * window, 
                                         const gchar * file,
@@ -778,6 +822,8 @@ move_to_trash_cb (GtkAction * action,
 			gtk_tree_selection_unselect_iter (GTK_TREE_SELECTION (gsearch->search_results_selection), &iter);
 
 			if (result == GNOME_VFS_OK) {
+				tree_model_iter_free_monitor (GTK_TREE_MODEL (gsearch->search_results_list_store),
+							      NULL, &iter, NULL);
 				gtk_list_store_remove (GTK_LIST_STORE (gsearch->search_results_list_store), &iter);
 			}
 			else {
@@ -810,6 +856,8 @@ move_to_trash_cb (GtkAction * action,
 				}
 
 				if (result == GNOME_VFS_OK) {
+					tree_model_iter_free_monitor (GTK_TREE_MODEL (gsearch->search_results_list_store),
+								      NULL, &iter, NULL);
 					gtk_list_store_remove (GTK_LIST_STORE (gsearch->search_results_list_store), &iter);
 				}
 				else {
@@ -980,6 +1028,91 @@ file_event_after_cb  (GtkWidget * widget,
 	return FALSE;
 }
 
+gboolean
+file_motion_notify_cb (GtkWidget *widget,
+                       GdkEventMotion *event,
+                       gpointer user_data)
+{
+        GSearchWindow * gsearch = user_data;
+	GdkCursor * cursor;
+	GtkTreePath * last_hover_path;
+	GtkTreeIter iter;
+ 	
+	if (gsearch->is_search_results_single_click_to_activate == FALSE) {
+		gdk_window_set_cursor (event->window, NULL);
+		return FALSE;
+	}
+	
+	if (event->window != gtk_tree_view_get_bin_window (GTK_TREE_VIEW (gsearch->search_results_tree_view))) {
+		gdk_window_set_cursor (event->window, NULL);
+                return FALSE;
+	}
+	
+	last_hover_path = gsearch->search_results_hover_path;
+
+	gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
+				       event->x, event->y,
+				       &gsearch->search_results_hover_path,
+				       NULL, NULL, NULL);
+
+	if (gsearch->search_results_hover_path != NULL) { 
+		cursor = gdk_cursor_new (GDK_HAND2);
+	}
+	else { 
+		cursor = NULL;
+	}
+	
+	gdk_window_set_cursor (event->window, cursor);
+
+	/* Redraw if the hover row has changed */
+	if (!(last_hover_path == NULL && gsearch->search_results_hover_path == NULL) &&
+	    (!(last_hover_path != NULL && gsearch->search_results_hover_path != NULL) ||
+	     gtk_tree_path_compare (last_hover_path, gsearch->search_results_hover_path))) {
+		if (last_hover_path) {
+			gtk_tree_model_get_iter (GTK_TREE_MODEL (gsearch->search_results_list_store),
+			                         &iter, last_hover_path);
+			gtk_tree_model_row_changed (GTK_TREE_MODEL (gsearch->search_results_list_store),
+			                            last_hover_path, &iter);
+		}
+		
+		if (gsearch->search_results_hover_path) {
+			gtk_tree_model_get_iter (GTK_TREE_MODEL (gsearch->search_results_list_store),
+			                         &iter, gsearch->search_results_hover_path);
+			gtk_tree_model_row_changed (GTK_TREE_MODEL (gsearch->search_results_list_store),
+			                            gsearch->search_results_hover_path, &iter);
+		}
+	}
+	
+	gtk_tree_path_free (last_hover_path);
+
+ 	return FALSE;
+}
+
+gboolean 
+file_leave_notify_cb (GtkWidget *widget,
+                      GdkEventCrossing *event,
+                      gpointer user_data)
+{
+        GSearchWindow * gsearch = user_data;
+	GtkTreeIter iter;
+
+	if (gsearch->is_search_results_single_click_to_activate && (gsearch->search_results_hover_path != NULL)) {
+		gtk_tree_model_get_iter (GTK_TREE_MODEL (gsearch->search_results_list_store),
+		                         &iter, 
+		                         gsearch->search_results_hover_path);
+		gtk_tree_model_row_changed (GTK_TREE_MODEL (gsearch->search_results_list_store),
+		                            gsearch->search_results_hover_path,
+		                            &iter);
+
+		gtk_tree_path_free (gsearch->search_results_hover_path);
+		gsearch->search_results_hover_path = NULL;
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 void  
 drag_begin_file_cb (GtkWidget * widget,
                     GdkDragContext * context,
@@ -1168,6 +1301,9 @@ show_file_selector_cb (GtkAction * action,
 	                                            GTK_STOCK_SAVE, GTK_RESPONSE_OK,
 	                                            NULL);
 
+#if GTK_CHECK_VERSION(2,7,3)
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (file_chooser), TRUE);
+#endif
 	if (gsearch->save_results_as_default_filename != NULL) {
 		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (file_chooser),
 		                               gsearch->save_results_as_default_filename);
@@ -1243,6 +1379,7 @@ display_dialog_could_not_save_to (GtkWidget * window,
 	g_free (primary);
 }
 
+#if !GTK_CHECK_VERSION(2,7,3)
 static gint
 display_dialog_could_not_save_exists (GtkWidget * window,
                                       const gchar * file)
@@ -1287,6 +1424,7 @@ display_dialog_could_not_save_exists (GtkWidget * window,
 
 	return response;
 }
+#endif
 
 void
 save_results_cb (GtkWidget * chooser,
@@ -1326,6 +1464,7 @@ save_results_cb (GtkWidget * chooser,
 		return;	
 	}
 	
+#if !GTK_CHECK_VERSION(2,7,3)	
 	if (g_file_test (gsearch->save_results_as_default_filename, G_FILE_TEST_EXISTS)) {
 
 		gint response;
@@ -1337,18 +1476,16 @@ save_results_cb (GtkWidget * chooser,
 			return;
 		}
 	}
+#endif	
 
 	if ((fp = fopen (gsearch->save_results_as_default_filename, "w")) != NULL) {
 
-		if (gtk_tree_model_get_iter_root (GTK_TREE_MODEL (store), &iter)) {
+		gint index;
 
-			gint n_children;
-			gint index;
+		for (index = 0; index < gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL); index++)
+		{
+			if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (store), &iter, NULL, index) == TRUE) {
 
-			n_children = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (store), NULL);
-
-			for (index = 0; index < n_children; index++)
-			{
 				gchar * utf8_path;
 				gchar * utf8_name;
 				gchar * utf8_file;
@@ -1356,7 +1493,6 @@ save_results_cb (GtkWidget * chooser,
 
 				gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COLUMN_PATH, &utf8_path, -1);
 				gtk_tree_model_get (GTK_TREE_MODEL (store), &iter, COLUMN_NAME, &utf8_name, -1);
-				gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter);
 
 				utf8_file = g_build_filename (utf8_path, utf8_name, NULL);			    
 				locale_file = g_filename_from_utf8 (utf8_file, -1, NULL, NULL, NULL);
@@ -1462,3 +1598,49 @@ disable_quick_search_cb (GtkWidget * dialog,
 	}
 }
 
+void
+single_click_to_activate_key_changed_cb (GConfClient * client, 
+                                         guint cnxn_id,
+                                         GConfEntry * entry,
+                                         gpointer user_data)
+{
+	GSearchWindow * gsearch = user_data;
+	GConfValue * value;
+
+	value = gconf_entry_get_value (entry);
+
+	g_return_if_fail (value->type == GCONF_VALUE_STRING);
+
+	gsearch->is_search_results_single_click_to_activate = 
+		(strncmp (gconf_value_get_string (value), "single", 6) == 0) ? TRUE : FALSE;
+}
+
+void
+columns_changed_cb (GtkTreeView * treeview,
+                    gpointer user_data)
+{
+	GSList * order;
+
+	order = gsearchtool_get_columns_order (treeview);
+
+	if (g_slist_length (order) == NUM_VISIBLE_COLUMNS) {
+		gsearchtool_gconf_set_list ("/apps/gnome-search-tool/columns_order", order, GCONF_VALUE_INT);
+	}
+	g_slist_free (order);
+}
+
+gboolean
+window_state_event_cb (GtkWidget * widget, 
+                       GdkEventWindowState * event, 
+                       gpointer data)
+{
+	GSearchWindow * gsearch = data;
+	
+	if (event->new_window_state & GDK_WINDOW_STATE_MAXIMIZED) {
+		gsearch->is_window_maximized = TRUE;
+	}
+	else {
+		gsearch->is_window_maximized = FALSE;
+	}
+	return FALSE;
+}
