@@ -35,8 +35,7 @@
 #include <stdlib.h>
 
 static gboolean queue_err_messages = FALSE;
-static GList *msg_queue_main = NULL, *msg_queue_sec = NULL;
-extern gboolean restoration_complete;
+static GSList *msg_queue_main = NULL, *msg_queue_sec = NULL;
 const char *month[12] =
 {N_("January"), N_("February"), N_("March"), N_("April"), N_("May"),
  N_("June"), N_("July"), N_("August"), N_("September"), N_("October"),
@@ -62,8 +61,8 @@ void
 error_dialog_show (GtkWidget *window, char *main, char *secondary)
 {
     if (queue_err_messages) {
-        msg_queue_main = g_list_append (msg_queue_main, g_strdup (main));
-        msg_queue_sec = g_list_append (msg_queue_sec, g_strdup (secondary));
+        msg_queue_main = g_slist_append (msg_queue_main, g_strdup (main));
+        msg_queue_sec = g_slist_append (msg_queue_sec, g_strdup (secondary));
     } else
         error_dialog_run (window, main, secondary);
 }
@@ -79,11 +78,11 @@ error_dialog_show_queued (void)
 {
 	if (msg_queue_main != NULL) {
 		gboolean title_created = FALSE;
-		GList *li, *li_sec;
+		GSList *li, *li_sec;
 		GString *gs = g_string_new (NULL);
 		GString *gs_sec = g_string_new (NULL);
 
-		for (li = msg_queue_main, li_sec=msg_queue_sec; li != NULL; li = li->next) {
+		for (li = msg_queue_main, li_sec=msg_queue_sec; li != NULL; li = g_slist_next (li)) {
 			char *msg = li->data;
 			char *sec = li_sec->data;
 
@@ -103,8 +102,8 @@ error_dialog_show_queued (void)
 			g_free (sec);
 			li_sec = li_sec->next;
 		}
-		g_list_free (msg_queue_main);
-		g_list_free (msg_queue_sec);
+		g_slist_free (msg_queue_main);
+		g_slist_free (msg_queue_sec);
 		msg_queue_main = NULL;
 		msg_queue_sec = NULL;
 
@@ -118,19 +117,17 @@ error_dialog_show_queued (void)
 char *
 locale_to_utf8 (const char *in)
 {
-	/* FIXME: we could guess the language from the line how we guessed
-	 * the month and can we get this to utf8 correctly even if it's not in
-	 * the current encoding */
-    char *out = g_locale_to_utf8 (in, -1, NULL, NULL, NULL);
+  char *out;
+  
+  if (g_utf8_validate (in, -1, NULL))
+	out = g_strdup (in);
+  else {
+	out = g_locale_to_utf8 (in, -1, NULL, NULL, NULL);
+	if (out == NULL)
+	  out = g_strdup ("?");
+  }
 
-	if (out == NULL) {
-		if (g_utf8_validate (in, -1, NULL))
-			return g_strdup (in);
-		else
-			return g_strdup ("?");
-	} else {
-		return out;
-	}
+  return out;
 }
 
 GDate *
@@ -144,8 +141,12 @@ string_get_date (char *line)
         return NULL;
 
     cp = strptime (line, "%b %d", &tp);
-    if (cp == 0)
-        return NULL;
+    if (cp == 0) {
+        cp = strptime (line, "%F", &tp);
+        if (cp == 0) {
+            return NULL;
+        }
+    }
 
     date = g_date_new_dmy (tp.tm_mday, tp.tm_mon+1, 70);
     return date;
